﻿using Falcon.SugarApi.DatabaseDefinitions.Cache;
using Falcon.SugarApi.JsonSerialize;
using Microsoft.Extensions.Caching.Distributed;
using SqlSugar;
using System;
using System.ComponentModel.DataAnnotations;
using System.Reflection;

namespace Falcon.SugarApi.DatabaseDefinitions
{
    /// <summary>
    /// SqlSugar数据库连接配置
    /// </summary>
    public class SugarConnectionConfig : ConnectionConfig
    {
        /// <summary>
        /// 是否使用log
        /// </summary>
        public bool Log { get; set; }

        /// <summary>
        /// 实例化SugarDb链接配置
        /// </summary>
        public SugarConnectionConfig() {
            this.ConfigureExternalServices ??= new ConfigureExternalServices { };
            //设置Nullable
            this.ConfigureExternalServices.EntityService += (p, c) => {
                var pt = p.PropertyType;
                //所有类型默认可空
                bool na = true;
                //字符串默认可空
                na = pt == typeof(string) ? true : na;
                //Nullable<>类型可空
                if (pt.IsGenericType && pt.GetGenericTypeDefinition() == typeof(Nullable<>)) {
                    na = true;
                }
                //RequiredAttribute标记不可空
                if (p.GetCustomAttribute<RequiredAttribute>() != null) {
                    na = false;
                }
                //主键不可以为空
                if (p.TryGetAttribute<KeyAttribute>(out var _)) {
                    na = false;
                }
                //定义主键不可以为空
                if (p.TryGetAttribute<SugarColumn>(out var sc) && sc.IsPrimaryKey) {
                    na = false;
                }
                c.IsNullable = na;
                //var sc = pt.GetCustomAttribute<SugarColumn>();
                //if (sc != null) {
                //    c.IsNullable = sc.IsNullable;
                //}
                //var isNullableTypes = new Type[] { typeof(string) };
                //if (isNullableTypes.Contains(pt)) {
                //    c.IsNullable = true;
                //}
                //else {
                //    c.IsNullable = false;
                //}
                //if (pt == typeof(string) && pt.GetCustomAttribute<RequiredAttribute>() == null) {
                //    c.IsNullable = true;
                //}
            };
        }

        private static ICacheService? CacheService = null;

        /// <summary>
        /// 通过配置实现Redis缓冲，必须单例实现.
        /// <para>默认:127.0.0.1:6379,password=,connectTimeout=3000,connectRetry=1,syncTimeout=10000,DefaultDatabase=0</para>
        /// </summary>
        public void AddRedisCache() {
            this.ConfigureExternalServices ??= new ConfigureExternalServices { };
            this.ConfigureExternalServices.DataInfoCacheService = new SqlSugarRedisCache();
        }

        /// <summary>
        /// 通过配置实现Redis缓冲，必须单例实现.
        /// </summary>
        public void AddRedisCache(string connectionString) {
            if (connectionString.IsNullOrEmpty()) {
                AddRedisCache();
                return;
            }
            this.ConfigureExternalServices ??= new ConfigureExternalServices { };
            this.ConfigureExternalServices.DataInfoCacheService = new SqlSugarRedisCache(connectionString);
        }

        /// <summary>
        /// HttpRuntimeCache 缓存
        /// </summary>
        public void AddHttpRuntimeCache() {
            this.ConfigureExternalServices ??= new ConfigureExternalServices { };
            this.ConfigureExternalServices.DataInfoCacheService = new HttpRuntimeCache();
        }

        /// <summary>
        /// 分布式缓存
        /// </summary>
        /// <param name="cache">缓存提供程序</param>
        /// <param name="serialize">序列化实现</param>
        public void AddDistributedCache(IDistributedCache cache, IJsonSerialize serialize) {
            this.ConfigureExternalServices ??= new ConfigureExternalServices { };
            this.ConfigureExternalServices.DataInfoCacheService = new DistributedCache(cache,serialize);
        }
    }
}
